#-------------------------------------------------------------------------------------------------------------------------------
# Copyright (c) 2018 Quest Software
#
# Name        :  SchemaCompareSample.psq
# Description :  Script which can be used to perform schema comparisons using Toad DevOps Toolkit.
#-------------------------------------------------------------------------------------------------------------------------------


Param (
  [Parameter(Mandatory = $True)]
  [String] $Source,                 # Source Oracle Connection string 
  [Parameter(Mandatory = $True)]
  [String] $Target,		            # Target Oracle Connection string 

  [Parameter(Mandatory = $True)]
  [String[]] $SourceSchemas,		# List of source schemas to compare
  [Parameter(Mandatory = $False)]
  [String[]] $TargetSchemas,		# List of target schemas to compare
                                    # If no TargetSchemas are given, then the comparison will use the same schema name for both
                                    # If TargetSchemas are provided, it must be the same size as the SourceSchemas list.  For the compare TDT will
                                    # will compare the first items of the lists, then the second items, and so on.
                                    # You can specify multiple schemas list "SCHEMA1","SCHEMA2".
                                    # NOTE:  wildcards can not be used when specifying schemas!
                                    
 
  [String] $OutputPath,				# Output path of any additional reports
  [Switch] $HTML,	                # Produce HTML with the comparison results
  [Switch] $GenerateChangeScript	# Produce SQL script to apply to source changes to the target
)
    
# Used for removing any invalid file name characters
Function Remove-InvalidFileNameChars {
  Param (
    [Parameter(Mandatory = $True,
      Position = 0,
      ValueFromPipeline = $True,
      ValueFromPipelineByPropertyName = $True)]
    [String] $Name
  )

  $InvalidChars = [IO.Path]::GetInvalidFileNameChars() -join ''
  $RegEx = "[{0}]" -f [RegEx]::Escape($InvalidChars)
  Return $Name -replace $RegEx
}           



# Start TDT
$TDT = New-Object -ComObject 'Toad.ToadAutoObject'

try {

    # Set the source/target connections
	$SourceConnection = $TDT.Connections.NewConnection($Source)
    $TDT.CompareSchemas.Source.Connection = $SourceConnection           
	$TargetConnection = $TDT.Connections.NewConnection($Target)
    $TDT.CompareSchemas.Target.Connection = $TargetConnection             

    # if no TargetSchemas list is given, make is the same as the SourceSchemas list
    if ($TargetSchemas.Count -eq 0) {
        $TargetSchemas = $SourceSchemas
    }

    # Make sure that the two schema lists are the same size
    if ($SourceSchemas.Count -ne $TargetSchemas.Count) {
        
        # Invalid lists given
		Write-Error "The Source and Target schema lists are not the same size!"
    }

    # Create the output path if it doesn't exist
    if ($OutputPath) {
        New-Item -itemType Directory -force -path $OutputPath | Out-Null
    }
	
	# Import any specified unit test files
	ForEach($SourceSchema in $SourceSchemas) {

        $TargetSchema = $TargetSchemas.Get($SourceSchemas.IndexOf($SourceSchema))

        Write-Output ("Comparing schema '{0}' with '{1}'" -f $SourceSchema, $TargetSchema)

        # Now compare the schemas
        $TDT.CompareSchemas.Source.Schema     = $SourceSchema    
        $TDT.CompareSchemas.Target.Schema     = $TargetSchema  

        $TDT.CompareSchemas.Execute()

        # Output the HTML comparison report, if specified
        if ($HTML) {
            
            # set the name of the change script
            $OutputFile = $SourceSchema + "-ChangeReport.html"
            $OutputFile = Remove-InvalidFileNameChars $OutputFile
            $OutputFile = Join-Path $OutputPath $OutputFile
            $TDT.CompareSchemas.GetDifferenceDetailsHTML() | Out-File $OutputFile
        }

        # Output the comparison script if specified
        if ($GenerateChangeScript) {
            
            # set the name of the change script
            $OutputFile = $SourceSchema + "-ChangeScript.sql"
            $OutputFile = Remove-InvalidFileNameChars $OutputFile
            $OutputFile = Join-Path $OutputPath $OutputFile
            $TDT.CompareSchemas.GetScript() | Out-File $OutputFile
        }
    }     

  } catch {
  Write-Error $_.Exception.Message
} finally {
  $TDT.Quit()
}
