#-------------------------------------------------------------------------------------------------------------------------------
# Copyright (c) 2018 Quest Software
#
# Name        :  UnitTestSample_TFS.psq
# Description :  Script which can be used by Microsoft Team Foundation Service to executes unit tests using Toad DevOps Toolkit.
#-------------------------------------------------------------------------------------------------------------------------------


Param (
  [String] $Connection,			# Oracle Connection string 
  [String[]] $Objects,			# List of database objects which have unit tests which need to be executed in the form of
							# OWNER.OBJECTNAME.  Wildcards can be used.  
							#     "SCHEMA.%","SCHEMA2.MyProc") - Execute all unit tests for the SCHEMA and for SCHEMA2.MyProc
  [String] $OutputPath,			# Output path of any additional reports
  [Switch] $TXT,				# Produce text based report
  [Switch] $XML					# Produce XML based report
)

# Used for removing any invalid file name characters
Function Remove-InvalidFileNameChars {
  Param (
    [Parameter(Mandatory = $True,
      Position = 0,
      ValueFromPipeline = $True,
      ValueFromPipelineByPropertyName = $True)]
    [String] $Name
  )

  $InvalidChars = [IO.Path]::GetInvalidFileNameChars() -join ''
  $RegEx = "[{0}]" -f [RegEx]::Escape($InvalidChars)
  Return $Name -replace $RegEx
}


# Start TDT
$TDT = New-Object -ComObject 'Toad.ToadAutoObject'

try {
	$TDT.UnitTesting.Connection = $TDT.Connections.NewConnection($Connection)

	# Set database objects
	ForEach($Object in $Objects) {
		$Owner, $Name = $Object.Split('.')

		$DBObject = $TDT.UnitTesting.UnitTests.DBObjects.Add()
		$DBObject.ObjectName  = $Name
		$DBObject.ObjectOwner = $Owner
	}  

  # Get the units tests for the object(s)
  $TDT.UnitTesting.UnitTests.Refresh()

  # Create the output path if it doesn't exist
  if ($OutputPath) {
    New-Item -itemType Directory -force -path $OutputPath | Out-Null
  }

  # Execute unit tests
  ForEach($UnitTest in $TDT.UnitTesting.UnitTests) {
    $UnitTest.Execute() | Out-Null

    # Report status of the execution.  If failed then display in RED 
    if ($UnitTest.LastRunStatus -ne 'SUCCESS') {
		Write-Output ("##vso[task.logissue type=error] Running... {0} Status:  {1}" -f $UnitTest.Name, $UnitTest.LastRunStatus)
		
		# if you want the execution of this script to fail the build, please remove the following commented out line
		#Write-Error "Error executing unit tests"
    }
	else
	{
		Write-Output ("Running... {0} Status: {1}" -f $UnitTest.Name, $UnitTest.LastRunStatus)
	}

	# See if should output additional reports
    if ($TXT) {
      $OutputFile = "{0} - {1}.txt" -f $UnitTest.Name, $UnitTest.LastRunStatus
      $OutputFile = Remove-InvalidFileNameChars $OutputFile
      $OutputFile = Join-Path $OutputPath $OutputFile
      $UnitTest.GetLastRunReportText() | Out-File $OutputFile
    }

    if ($XML) {
      $OutputFile = "{0} - {1}.xml" -f $UnitTest.Name, $UnitTest.LastRunStatus
      $OutputFile = Remove-InvalidFileNameChars $OutputFile
      $OutputFile = Join-Path $OutputPath $OutputFile
      $UnitTest.GetLastRunReportXML() | Out-File $OutputFile
    }
  }
} catch {
  Write-Error $_.Exception.Message
} finally {
  $TDT.Quit()
}
